@@ -7,7 +7,7 @@
 #' OMexample()
 #' }
 OMexample <- function() {
-  fromRMD <- system.file("Example_Chile_hake_source.Rmd", package="DLMtool")
+  fromRMD <- system.file("Example_Chile_Hake.Rmd", package="DLMtool")
   tt <- file.copy(fromRMD, getwd(), overwrite = TRUE)
   fromXL <- system.file("Example_Chile_hake.xlsx", package="DLMtool")
   tt <- file.copy(fromXL, getwd(), overwrite = TRUE)
@@ -25,24 +25,24 @@ OMexample <- function() {
 #' to use as templates for the Operating Model.
 #' @param overwrite Logical. Should files be overwritten if they already exist?
 #'
-#' @return name.xlsx and name_source.rmd files are created in the working directory.  
+#' @return name.xlsx and name.rmd files are created in the working directory.  
 #' @export
 #' @author A. Hordyk
 #'
 #' @examples
 #' \dontrun{
-#' # Create an Excel OM template and rmd file called 'myOM.xlsx' and 'myOM_source.rmd': 
+#' # Create an Excel OM template and rmd file called 'myOM.xlsx' and 'myOM.rmd': 
 #' OMinit('myOM')
 #' 
-#' # Create an Excel OM template and text file called 'myOM.rmd' and 'myOM_source.rmd', using
+#' # Create an Excel OM template and text file called 'myOM.rmd' and 'myOM.rmd', using
 #' another OM as a template: 
 #' OMinit('myOM', myOM)
 #' 
-#' # Create an Excel OM template and text file called 'myOM.rmd' and 'myOM_source.rmd', using
+#' # Create an Excel OM template and text file called 'myOM.rmd' and 'myOM.rmd', using
 #' the Stock object 'Herring' as a template: 
 #' OMinit('myOM', Herring)
 #' 
-#' # Create an Excel OM template and text file called 'myOM.rmd' and 'myOM_source.rmd', using
+#' # Create an Excel OM template and text file called 'myOM.rmd' and 'myOM.rmd', using
 #' the Stock object 'Herring', and Obs object 'Generic_obs' as templates: 
 #' OMinit('myOM', Herring, Generic_obs)
 #' }
@@ -269,7 +269,8 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), overwrite=FALSE) {
     
     
     # OM sheet####
-    df <- data.frame(Slot=c("Name", "nsim", "proyears", "interval", "pstar", "maxF", "reps"))
+    df <- data.frame(Slot=c("Name", "Agency", "Region", "Latitude", "Longitude",
+                            "nsim", "proyears", "interval", "pstar", "maxF", "reps"))
     
     # write slots 
     openxlsx::writeData(wb, sheet = "OM", x = df, 
@@ -290,7 +291,7 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), overwrite=FALSE) {
     
     df <- data.frame(Values=c( 48, 50, 4, 0.5, 0.8, 1))
     openxlsx::writeData(wb, sheet = "OM", x = df, 
-                        startCol = 2, startRow = 3,
+                        startCol = 2, startRow = 7,
                         colNames = FALSE, rowNames = FALSE, 
                         withFilter = FALSE,
                         keepNA = FALSE) 
@@ -331,8 +332,8 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), overwrite=FALSE) {
  
   if ('rmd' %in% files) {
     ## Write Rmd source skeleton ####
-    message("Creating ", nameNoExt, "_source.rmd in ", getwd())
-    RmdSource <- paste0(nameNoExt, "_source.rmd")
+    message("Creating ", nameNoExt, ".rmd in ", getwd())
+    RmdSource <- paste0(nameNoExt, ".rmd")
     if (file.exists(RmdSource) & !overwrite) {
       stop(RmdSource, " already exists.\n Use 'overwrite=TRUE'.", call.=FALSE)
     } else {
@@ -438,11 +439,13 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), overwrite=FALSE) {
     cat("\n\n# Stock Parameters\n\n", sep="", append=TRUE, file=RmdSource) 
     slots <- slotNames("Stock")
     for (X in slots) {
-      cat("## ", X, "\n", sep="", append=TRUE, file=RmdSource)
-      if (is.null(ObTemplates$Stock)) {
-        cat("No justification provided. \n\n", sep="", append=TRUE, file=RmdSource)  
-      } else {
-        cat("Borrowed from ", ObTemplates$Stock@Name, "\n\n", sep="", append=TRUE, file=RmdSource)
+      if (! X %in% c("Name", "Species")) {
+        cat("## ", X, "\n", sep="", append=TRUE, file=RmdSource)
+        if (is.null(ObTemplates$Stock)) {
+          cat("No justification provided. \n\n", sep="", append=TRUE, file=RmdSource)  
+        } else {
+          cat("Borrowed from ", ObTemplates$Stock@Name, "\n\n", sep="", append=TRUE, file=RmdSource)
+        }
       }
     }
     
@@ -450,12 +453,15 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), overwrite=FALSE) {
     cat("\n\n# Fleet Parameters\n\n", sep="", append=TRUE, file=RmdSource) 
     slots <- slotNames("Fleet")
     for (X in slots) {
-      cat("## ", X, "\n", sep="", append=TRUE, file=RmdSource)
-      if (is.null(ObTemplates$Fleet)) {
-        cat("No justification provided. \n\n", sep="", append=TRUE, file=RmdSource)  
-      } else {
-        cat("Borrowed from ", ObTemplates$Fleet@Name, "\n\n", sep="", append=TRUE, file=RmdSource)
+      if (! X %in% c("Name")) {
+        cat("## ", X, "\n", sep="", append=TRUE, file=RmdSource)
+        if (is.null(ObTemplates$Fleet)) {
+          cat("No justification provided. \n\n", sep="", append=TRUE, file=RmdSource)  
+        } else {
+          cat("Borrowed from ", ObTemplates$Fleet@Name, "\n\n", sep="", append=TRUE, file=RmdSource)
+        }
       }
+    
     }
     
     
@@ -463,24 +469,31 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), overwrite=FALSE) {
     cat("\n\n# Obs Parameters\n\n", sep="", append=TRUE, file=RmdSource) 
     slots <- slotNames("Obs")
     for (X in slots) {
-      cat("## ", X, "\n", sep="", append=TRUE, file=RmdSource)
-      if (is.null(ObTemplates$Obs)) {
-        cat("No justification provided. \n\n", sep="", append=TRUE, file=RmdSource)  
-      } else {
-        cat("Borrowed from ", ObTemplates$Obs@Name, "\n\n", sep="", append=TRUE, file=RmdSource)
+      if (! X %in% c("Name")) {
+        cat("## ", X, "\n", sep="", append=TRUE, file=RmdSource)
+        if (is.null(ObTemplates$Obs)) {
+          cat("No justification provided. \n\n", sep="", append=TRUE, file=RmdSource)  
+        } else {
+          cat("Borrowed from ", ObTemplates$Obs@Name, "\n\n", sep="", append=TRUE, file=RmdSource)
+        }
       }
+      
+
     }
     
     # Imp Parameters ####
     cat("\n\n# Imp Parameters\n\n", sep="", append=TRUE, file=RmdSource) 
     slots <- slotNames("Imp")
     for (X in slots) {
-      cat("## ", X, "\n", sep="", append=TRUE, file=RmdSource)
-      if (is.null(ObTemplates$Imp)) {
-        cat("No justification provided. \n\n", sep="", append=TRUE, file=RmdSource)  
-      } else {
-        cat("Borrowed from ", ObTemplates$Imp@Name, "\n\n", sep="", append=TRUE, file=RmdSource)
+      if (! X %in% c("Name")) {
+        cat("## ", X, "\n", sep="", append=TRUE, file=RmdSource)
+        if (is.null(ObTemplates$Imp)) {
+          cat("No justification provided. \n\n", sep="", append=TRUE, file=RmdSource)  
+        } else {
+          cat("Borrowed from ", ObTemplates$Imp@Name, "\n\n", sep="", append=TRUE, file=RmdSource)
+        }
       }
+    
     }
     
     # References ####
@@ -568,6 +581,7 @@ XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
   # Load the Excel File ####
   if (is.null(name)) {
     fls <- list.files(pattern=".xlsx", ignore.case = TRUE)
+    fls <- fls[!grepl('~', fls)]
     if (length(fls) == 0) stop('Name not provided and no .xlsx files found.', call.=FALSE)
     if (length(fls) > 1) stop("Name not provided and multiple .xlsx files found", call.=FALSE)
     name <- fls
@@ -620,17 +634,26 @@ XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
   
   # Read in the OM sheet
   sht <- as.data.frame(readxl::read_excel(name, sheet = "OM", col_names = FALSE))
-  dat <- sht[,1:2] 
+  dat <- sht # sht[,1:2] 
   dat <- dat[which(dat[,1] != "Slot"),]
   # if (ncol(sht)>2) warning("More than two columns found in Sheet OM. Values in columns C+ are ignored")
   if (ncol(sht)<2) {
     message("No values found for OM slots in Sheet OM. Using defaults")
   } else {
     for (xx in 1:nrow(dat)) {
-      val <- dat[xx, 2]
+      val <- dat[xx, 2:ncol(dat)]
       if (length(val)) {
-        if (dat[xx,1] != "Name") slot(OM, dat[xx, 1]) <- as.numeric(val)
-        if (dat[xx,1] == "Name") slot(OM, dat[xx, 1]) <- val
+        if (!dat[xx,1] %in% c("Name", "Agency", "Region")) {
+          options(warn=-1)
+          val <- as.numeric(val)
+          options(warn=1)
+          val <- val[!is.na(val)]
+          if (.hasSlot(OM, dat[xx,1])) slot(OM, dat[xx, 1]) <- val
+        } else  {
+          val <- val[!is.na(val)]
+          if (.hasSlot(OM, dat[xx,1])) slot(OM, dat[xx, 1]) <- val
+        }
+        
       } else{
         message("No value found for OM slot ", dat[xx,1], ". Using default: ", slot(OM, dat[xx, 1]))
       }
@@ -647,7 +670,7 @@ XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
   ChkObj(OM)
   if (msg) {
     message('OM successfully imported\n')
-    message("Document OM slots in source.rmd file (probably ", tools::file_path_sans_ext(name), "_source.rmd),
+    message("Document OM slots in .rmd file (probably ", tools::file_path_sans_ext(name), ".rmd),
   and run 'OMdoc' if OM parameter values have changed." )
   }
 
@@ -681,7 +704,7 @@ writeCSV2 <- function(inobj, tmpfile = NULL, objtype = c("Stock", "Fleet",
 #'
 #' @param OM An object of class 'OM' or the name of an OM xlsx file 
 #' @param rmd.source Optional. Name of the source.rmd file corresponding to the 'OM'. Default assumption
-#' is that the file is 'OM@Name_source.Rmd'
+#' is that the file is 'OM@Name.Rmd'
 #' @param overwrite Logical. Should existing files be overwritten?
 #' @param out.file Optional. Character. Name of the output file. Default is the same as the text file.
 #' @param inc.plot Logical. Should the plots be included?
@@ -714,6 +737,7 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
     OM <- XL2OM(OM, msg=FALSE)
   } else if (is.null(OM)) {
     fls <- list.files(pattern=".xlsx", ignore.case=TRUE)
+    fls <- fls[!grepl('~', fls)]
     if (length(fls)==1) OM <- XL2OM(fls, msg=FALSE)
     if (length(fls)>1) stop('argument "OM" not provided and multiple .xlsx files in working directory', call.=FALSE)
   } else stop('OM must be class "OM" or name of OM xlsx file.', call.=FALSE)
@@ -755,9 +779,14 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
     cat("This directory was created by DLMtool function OMdoc\n\n", sep="", append=TRUE, file='build/readme.txt') 
     cat("Files in this directory are used to generate the OM report.\n\n", sep="", append=TRUE, file='build/readme.txt') 
   } 
+  
+  if(dir.exists("images")) {
+    dir.create('build/images', showWarnings = FALSE)
+    cpy <- file.copy('images', 'build', overwrite=TRUE, recursive = TRUE)
+  }
 
   if (is.null(out.file)) out.file <- tools::file_path_sans_ext(rmd.source)
-  out.file <- gsub("_source", "_compiled", out.file)
+  # out.file <- gsub("_source", "_compiled", out.file)
   
   RMDfile <- paste0("build/", out.file, ".Rmd")
   # if (file.exists(RMDfile) & !overwrite) {
@@ -895,6 +924,7 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
         # check if OM has changed 
         changed <- rep(FALSE, length(slotNames(OM)))
         for (sl in seq_along(slotNames(OM))) {
+      
           oldOM <- slot(OM, slotNames(OM)[sl])
           newOM <- slot(testOM, slotNames(OM)[sl])
           if (class(oldOM) !='character') {
@@ -903,8 +933,14 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
               if (length(newOM)<1 || !is.finite(newOM)) newOM <- 0
               if (any(oldOM != newOM)) changed[sl] <- TRUE
             } else {
-              if (length(oldOM) != length(newOM)) changed[sl] <- TRUE
-              if (length(setdiff(oldOM, newOM)) > 0) changed[sl] <- TRUE
+              if (length(oldOM) != length(newOM)) {
+                changed[sl] <- TRUE
+              } else if (length(oldOM)>0){
+                for (xx in 1:length(oldOM)) {
+                  if(any(oldOM[[xx]] != newOM[[xx]]))changed[sl] <- TRUE
+                  
+                }
+              }
             }
           }
         }
@@ -948,15 +984,31 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
     textIn <- textIn[ind:length(textIn)]
   }
   
-  
+
   ## Introduction ####
   writeSection(class="Intro", OM, textIn, RMDfile, color=color, inc.plot=inc.plot)
   
   ## OM Details ####
   OMdesc <- DLMtool::OMDescription
-  cat("# OM Parameters \n", append=TRUE, file=RMDfile, sep="")
+  cat("# Operating Model \n", append=TRUE, file=RMDfile, sep="")
+  # Taxonomic Info and Location ####
+  if (.hasSlot(OM, 'Species') && !is.na(OM@Species)) {
+    cat("## Species Information \n\n", append=TRUE, file=RMDfile, sep="")
+    cat("**Species**: *", OM@Species, "*\n\n", append=TRUE, file=RMDfile, sep="")
+    cat("**Management Agency**: ", OM@Agency, "\n\n", append=TRUE, file=RMDfile, sep="")
+    cat("**Region**: ", OM@Region, "\n\n", append=TRUE, file=RMDfile, sep="")
+    if (length(OM@Latitude)>0) {
+      lat <- paste0(OM@Latitude, sep="", collapse=", ")
+      cat("**Latitude**: ", lat, "\n\n", append=TRUE, file=RMDfile, sep="")
+    }
+    if (length(OM@Longitude)>0) {
+      long <- paste0(OM@Longitude, sep="", collapse=", ")
+      cat("**Longitude**: ", long, "\n\n", append=TRUE, file=RMDfile, sep="")
+    }
+  }
   
-  cat("**Name**: ", OMdesc$Description[OMdesc$Slot =='Name'], ": ", "<span style='color:", color, "'>", " ", OM@Name, "</span>", "\n\n", append=TRUE, file=RMDfile, sep="")
+  cat("## OM Parameters \n", append=TRUE, file=RMDfile, sep="")
+  cat("**OM Name**: ", OMdesc$Description[OMdesc$Slot =='Name'], ": ", "<span style='color:", color, "'>", " ", OM@Name, "</span>", "\n\n", append=TRUE, file=RMDfile, sep="")
   
   cat("**nsim**: ", OMdesc$Description[OMdesc$Slot =='nsim'], ": ", "<span style='color:", color, "'>", " ", OM@nsim, "</span>", "\n\n", "\n\n", append=TRUE, file=RMDfile, sep="")
   
@@ -972,6 +1024,7 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
  
   cat("**Source**: ", OMdesc$Description[OMdesc$Slot =='Source'], "\n\n", "<span style='color:", color, "'>", " ", OM@Source, "</span>", "\n\n", append=TRUE, file=RMDfile, sep="")
   
+  
   useCpars <- length(OM@cpars) >0 
   ## Cpars ####
   if (useCpars) writeSection(class="cpars", OM, textIn, RMDfile, color=color, 
@@ -992,7 +1045,7 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
   ## OM Plots ####
   if (inc.plot) {
     cat("# OM Plots\n\n", sep="", append=TRUE, file=RMDfile) # write heading
-    cat("```{r plot.OM, echo=FALSE, fig.asp=2}\n", append=TRUE, file=RMDfile, sep="")
+    cat("```{r plotOM, echo=FALSE, fig.asp=2}\n", append=TRUE, file=RMDfile, sep="")
     cat("plot.OM(out)\n", append=TRUE, file=RMDfile, sep="")
     cat("```\n\n\n", append=TRUE, file=RMDfile, sep="")
 
@@ -1069,7 +1122,7 @@ Template <- function(type=c("Stock", "Fleet", "Obs", "Imp")) {
   Slots <- names(methods::getSlots(type))
   for (X in Slots) {
     tt <- grep(paste0("\\<", X, "\\>"), mat) 
-    if (X != "Name" && X != "Source") {
+    if (X != "Name" && X != "Source" && X!="Species" && X!="Latitude" && X!='Longitude') {
       if (length(tt) < 1) stop("slot ", X, " not found in ", type, " template", call.=FALSE)
       if (length(tt) > 1) stop("slot ", X, " found multiple times in ", type, " template", call.=FALSE)    
     }
@@ -1128,7 +1181,8 @@ writeSection <- function(class=c("Intro", "Stock", "Fleet", "Obs", "Imp", "Refer
       for (ll in (st+1):(end - 1)) {
         cat(textIn[ll], "\n\n", sep="", append=TRUE, file=RMDfile) # write cpars paragraphs
       }
-    }  else stop("More than one section # Custom Parameters", call.=FALSE)
+    } 
+    if (length(cparstext) > 1) stop("More than one section # Custom Parameters", call.=FALSE)
     
   } else {
     # Write class heading 
@@ -1153,6 +1207,13 @@ writeSection <- function(class=c("Intro", "Stock", "Fleet", "Obs", "Imp", "Refer
       stop("Invalid second level headings (must match slots in class ", class, "): ", paste(invalid, ""), call.=FALSE)
     }
     
+    # check for text after class heading 
+    preText <- textIn[bg:(bg+scLHloc[1]-2)]
+    if (any(nchar(preText)>0)) {
+      cat(preText, "\n\n", append=TRUE, file=RMDfile, sep="")
+    }
+  
+    
     # Get template for class section 
     ClTemp <- Template(class)
     
@@ -1169,14 +1230,13 @@ writeSection <- function(class=c("Intro", "Stock", "Fleet", "Obs", "Imp", "Refer
           # get slot value if not in cpars 
           if (useCpars && sl %in% names(OM@cpars)) {
             val <- range(OM@cpars[[sl]])
+            val <- signif(val,3)
             used <- TRUE
             val <- gsub('"', "", paste(val, collapse="\", \""))
-            valtext <- paste0("(", trimws(val), ")")
-            # currently not used. Manually describe in the custom parameters section
-            
+            valtext <- paste0("Specified in cpars: ", "<span style='color:", color, "'>", " ", trimws(val), "</span>", "\n\n")
           } else {
             val <- slot(OM, sl)
-            if (is.numeric(val)) val <- round(val,2)
+            if (is.numeric(val)) val <- signif(val,3)
             used <- length(val)>0 && !is.na(val) && !is.null(val) # is the slot used?
             if (used) {
               val <- gsub('"', "", paste(val, collapse="\", \""))
